@@ -3,7 +3,6 @@ function get_table as well as a set of predefined CIM queries.
 """
 
 import re
-from abc import ABC, abstractmethod
 from datetime import datetime
 from functools import cached_property
 from typing import Dict, Iterable, List, Optional, Tuple, Union
@@ -25,23 +24,23 @@ from cimsparql.enums import (
     SyncVars,
     TapChangerObjects,
 )
+from cimsparql.graphdb import GraphDBClient
 from cimsparql.type_mapper import TypeMapper
 from cimsparql.typehints import Region
-from cimsparql.url import Prefix
+from cimsparql.url import service
 
 
-class Model(Prefix, ABC):
-    def __init__(self, mapper: Optional[TypeMapper], *args, **kwargs) -> None:
-        super().__init__(*args, **kwargs)
+class Model:
+    def __init__(self, mapper: Optional[TypeMapper], client: GraphDBClient) -> None:
         self._mapper = mapper
+        self.client = client
 
-    @abstractmethod
-    def _get_table(self, *args, **kwargs) -> Tuple[pd.DataFrame, Dict[str, str]]:
-        pass
-
-    @abstractmethod
-    def _col_map(self) -> Dict[str, str]:
-        pass
+    @staticmethod
+    def _col_map(data_row) -> Dict[str, str]:
+        return {
+            column: data.get("datatype", data.get("type", None))
+            for column, data in data_row.items()
+        }
 
     @cached_property
     def mapper(self) -> Optional[TypeMapper]:
@@ -56,21 +55,6 @@ class Model(Prefix, ABC):
             return TypeMapper(self)
         return self._mapper
 
-    def _query_with_header(self, query: str, limit: Optional[int] = None) -> str:
-        query = "\n".join([self.header_str(query), query])
-        if limit is not None:
-            query += f" limit {limit}"
-        return query
-
-    @property
-    def empty(self) -> bool:
-        """Identify empty GraphDB repo"""
-        try:
-            self._get_table("SELECT * \n WHERE { ?s ?p ?o }", limit=1)
-            return False
-        except IndexError:
-            return True
-
     @staticmethod
     def _assign_column_types(
         result: pd.DataFrame, columns: Dict[str, Union[bool, str, float, int]]
@@ -91,41 +75,21 @@ class Model(Prefix, ABC):
         col_map.update(columns)
         return col_map
 
-    def get_table(
+    @property
+    def map_data_types(self) -> bool:
+        pref = self.client.prefixes.prefixes
+        return "cim" in pref and self.mapper.have_cim_version(pref["cim"])
+
+    def get_table_and_convert(
         self,
         query: str,
-        index: Optional[str] = None,
         limit: Optional[int] = None,
-        map_data_types: bool = True,
-        columns: Optional[Dict] = None,
+        index: Optional[str] = None,
+        columns: Optional[Dict[str, str]] = None,
     ) -> pd.DataFrame:
-        """Gets given table from the configured database.
+        result, data_row = self.client.get_table(query, limit)
 
-        Args:
-           query: to sparql server
-           index: column name to use as index
-           limit: limit number of resulting rows
-           map_data_types: gets datatypes from the configured graphdb & maps the types in the result
-                  to correct python types
-           custom_maps: dictionary of 'sparql_datatype': function to apply on columns with that
-                  type. Overwrites sparql map for the types specified.
-           columns: dictionary of 'column_name': function,
-                  uses pandas astype on the column, or applies function.
-                  Sparql map overwrites columns when available
-
-        Example:
-           >>> from cimsparql.graphdb import GraphDBClient
-           >>> from cimsparql.url import service
-           >>> gdbc = GraphDBClient(service('LATEST'))
-           >>> query = 'select * where { ?subject ?predicate ?object }'
-           >>> gdbc.get_table(query, limit=10)
-        """
-        try:
-            result, data_row = self._get_table(query, limit)
-        except IndexError:
-            return pd.DataFrame([])
-
-        if map_data_types and self.mapper is not None:
+        if self.mapper is not None:
             col_map = self.col_map(data_row, columns)
             result = self.mapper.map_data_types(result, col_map)
 
@@ -133,53 +97,24 @@ class Model(Prefix, ABC):
             result.set_index(index, inplace=True)
         return result
 
-    @property
-    def map_data_types(self) -> bool:
-        return "cim" in self.prefixes and self.mapper.have_cim_version(self.prefixes["cim"])
-
-    @classmethod
-    def _manual_convert_types(
-        cls, df: pd.DataFrame, columns: Optional[Dict], index: Optional[str]
-    ) -> pd.DataFrame:
-        if columns is None:
-            columns = {}
-        reset_index = index is not None
-        if reset_index:
-            df.reset_index(inplace=True)
-        df = df.astype(str)
-        cls._assign_column_types(df, columns)
-        if reset_index:
-            df.set_index(index, inplace=True)
-        return df
-
-    def _get_table_and_convert(
-        self,
-        query: str,
-        limit: Optional[int] = None,
-        index: Optional[str] = None,
-        columns: Optional[Dict[str, str]] = None,
-    ) -> pd.DataFrame:
-        result = self.get_table(query, index, limit, map_data_types=True, columns=columns)
-
-        if not self.map_data_types and len(result) > 0:
-            result = self._manual_convert_types(result, columns, index)
-
-        return result
-
 
 class CimModel(Model):
     """Used to query with sparql queries (typically CIM)"""
 
-    def __init__(self, *args, **kwargs) -> None:
-        super().__init__(*args, **kwargs)
+    def __init__(self, mapper: Optional[TypeMapper], client: GraphDBClient) -> None:
+        super().__init__(mapper, client)
         self._date_version = None
 
+    @property
+    def cim_version(self) -> int:
+        return self.client.prefixes.cim_version
+
     @property
     def date_version(self) -> datetime:
         """Activation date for this repository"""
         if self._date_version:
             return self._date_version
-        repository_date = self._get_table_and_convert(queries.version_date())
+        repository_date = self.get_table_and_convert(queries.version_date())
         self._date_version = repository_date["activationDate"].values[0]
         if isinstance(self._date_version, np.datetime64):
             self._date_version = self._date_version.astype("<M8[s]").astype(datetime)
@@ -188,8 +123,8 @@ class CimModel(Model):
     def full_model(self, dry_run: bool = False) -> Union[pd.DataFrame, str]:
         query = queries.full_model()
         if dry_run:
-            return self._query_with_header(query)
-        return self._get_table_and_convert(query)
+            return self.client.query_with_header(query)
+        return self.get_table_and_convert(query)
 
     def bus_data(
         self,
@@ -198,6 +133,7 @@ class CimModel(Model):
         limit: Optional[int] = None,
         dry_run: bool = False,
         with_dummy_buses: bool = False,
+        with_market: bool = True,
     ) -> Union[pd.DataFrame, str]:
         """Query name of topological nodes (TP query).
 
@@ -207,7 +143,7 @@ class CimModel(Model):
            limit: return first 'limit' number of rows
            dry_run: return string with sql query
         """
-        query = queries.bus_data(region, sub_region)
+        query = queries.bus_data(region, sub_region, with_market)
         if with_dummy_buses:
             dummy_bus_query = queries.three_winding_dummy_bus(region, sub_region)
             combined = sup.combine_statements(query, dummy_bus_query, split=union_split)
@@ -216,8 +152,9 @@ class CimModel(Model):
                 f"where {{{{{combined}}}}}",
             )
         if dry_run:
-            return self._query_with_header(query, limit)
-        return self._get_table_and_convert(query, limit, index="mrid")
+            return self.client.query_with_header(query, limit)
+        df = self.get_table_and_convert(query, limit)
+        return df.groupby("mrid").first()
 
     def phase_tap_changers(
         self,
@@ -234,8 +171,8 @@ class CimModel(Model):
             region, sub_region, with_tap_changer_values, impedance, tap_changer_objects
         )
         if dry_run:
-            return self._query_with_header(query, limit)
-        return self._get_table_and_convert(query, limit, index="mrid")
+            return self.client.query_with_header(query, limit)
+        return self.get_table_and_convert(query, limit, index="mrid")
 
     def loads(
         self,
@@ -245,10 +182,10 @@ class CimModel(Model):
         sub_region: bool = False,
         connectivity: Optional[Optional[str]] = None,
         nodes: Optional[Optional[str]] = None,
-        station_group_optional: bool = True,
         station_group: bool = False,
         with_sequence_number: bool = False,
         network_analysis: bool = True,
+        with_bidzone: bool = True,
         limit: Optional[int] = None,
         dry_run: bool = False,
         ssh_graph: Optional[str] = None,
@@ -256,26 +193,28 @@ class CimModel(Model):
         """Query load data
 
         Args:
-           load_type: List of load types. Allowed: "ConformLoad", "NonConformLoad", "EnergyConsumer"
-           load_vars: List of additional load vars to return. Possible are 'p' and/or 'q'.
-           region: Limit to region
-           sub_region: Assume region is a sub_region
-           connectivity: Include connectivity mrids
-           station_group: return station group mrid (if any)
-           with_sequence_number: Include the sequence numbers in output
-           network_analysis: Include only network analysis enabled components
-           limit: return first 'limit' number of rows
-           dry_run: return string with sql query
+            load_type: List of load types. Allowed: "ConformLoad", "NonConformLoad",
+                "EnergyConsumer"
+            load_vars: List of additional load vars to return. Possible are 'p' and/or 'q'.
+            region: Limit to region
+            sub_region: Assume region is a sub_region
+            connectivity: Include connectivity mrids
+            station_group: return station group mrid (if any)
+            with_sequence_number: Include the sequence numbers in output
+            network_analysis: Include only network analysis enabled components
+            with_bidzone: If True bidzone information is added, otherwise it is omitted
+            limit: return first 'limit' number of rows
+            dry_run: return string with sql query
 
         Returns:
            DataFrame: with mrid as index and columns ['terminal_mrid', 'bid_marked_code', 'p', 'q',
            'station_group']
 
         Example:
-           >>> from cimsparql.graphdb import GraphDBClient
-           >>> from cimsparql.url import service
-           >>> gdbc = GraphDBClient(service('LATEST'))
-           >>> gdbc.loads(load_type=['ConformLoad', 'NonConformLoad'])
+           >>> from cimsparql.model import get_cim_model
+           >>> server_url = "127.0.0.1:7200"
+           >>> model = get_cim_model(server_url, "LATEST")
+           >>> model.loads(load_type=['ConformLoad', 'NonConformLoad'])
         """
         query = queries.load_query(
             load_type,
@@ -284,17 +223,17 @@ class CimModel(Model):
             sub_region,
             connectivity,
             nodes,
-            station_group_optional,
+            station_group,
             with_sequence_number,
             network_analysis,
-            station_group,
             self.cim_version,
+            with_bidzone,
             ssh_graph,
         )
         if dry_run:
-            return self._query_with_header(query, limit)
+            return self.client.query_with_header(query, limit)
 
-        return self._get_table_and_convert(query, limit, index="mrid")
+        return self.get_table_and_convert(query, limit, index="mrid")
 
     def wind_generating_units(
         self,
@@ -314,16 +253,16 @@ class CimModel(Model):
            'maxP', 'allocationMax', 'allocationWeight', 'minP', 'name', 'power_plant_mrid']
 
         Example:
-           >>> from cimsparql.graphdb import GraphDBClient
-           >>> from cimsparql.url import service
-           >>> gdbc = GraphDBClient(service('LATEST'))
-           >>> gdbc.wind_generating_units(limit=10)
+            >>> from cimsparql.model import get_cim_model
+            >>> server_url = "127.0.0.1:7200"
+            >>> model = get_cim_model(server_url, "LATEST")
+            >>> model.wind_generating_units(limit=10)
 
         """
         query = queries.wind_generating_unit_query(network_analysis)
         if dry_run:
-            return self._query_with_header(query, limit)
-        return self._get_table_and_convert(query, limit, index="mrid")
+            return self.client.query_with_header(query, limit)
+        return self.get_table_and_convert(query, limit, index="mrid")
 
     def synchronous_machines(
         self,
@@ -332,12 +271,13 @@ class CimModel(Model):
         sub_region: bool = False,
         connectivity: Optional[str] = None,
         nodes: Optional[str] = None,
-        station_group_optional: bool = True,
+        station_group: bool = True,
         with_sequence_number: bool = False,
         network_analysis: bool = True,
         u_groups: bool = False,
         limit: Optional[int] = None,
         dry_run: bool = False,
+        with_market: bool = True,
         ssh_graph: Optional[str] = None,
     ) -> Union[pd.DataFrame, str]:
         """Query synchronous machines
@@ -347,7 +287,7 @@ class CimModel(Model):
            region: Limit to region
            sub_region: Assume region is a sub_region
            connectivity: Include connectivity mrids
-           station_group_optional: Assume station group is optional
+           station_group: Assume station group is optional
            with_sequence_number: add this numbers
            network_analysis: query SN:Equipment.networkAnalysisEnable
            u_groups: Filter out station groups where name starts with 'U-'
@@ -355,10 +295,10 @@ class CimModel(Model):
            dry_run: return string with sql query
 
         Example:
-           >>> from cimsparql.graphdb import GraphDBClient
-           >>> from cimsparql.url import service
-           >>> gdbc = GraphDBClient(service('LATEST'))
-           >>> gdbc.synchronous_machines(limit=10)
+            >>> from cimsparql.model import get_cim_model
+            >>> server_url = "127.0.0.1:7200"
+            >>> model = get_cim_model(server_url, "LATEST")
+            >>> model.synchronous_machines(limit=10)
         """
         query = queries.synchronous_machines_query(
             sync_vars,
@@ -366,16 +306,17 @@ class CimModel(Model):
             sub_region,
             connectivity,
             nodes,
-            station_group_optional,
+            station_group,
             self.cim_version,
             with_sequence_number,
             network_analysis,
             u_groups,
+            with_market,
             ssh_graph,
         )
         if dry_run:
-            return self._query_with_header(query, limit)
-        return self._get_table_and_convert(query, limit, index="mrid")
+            return self.client.query_with_header(query, limit)
+        return self.get_table_and_convert(query, limit, index="mrid")
 
     def connections(
         self,
@@ -401,17 +342,17 @@ class CimModel(Model):
            dry_run: return string with sql query
 
         Example:
-           >>> from cimsparql.graphdb import GraphDBClient
-           >>> from cimsparql.url import service
-           >>> gdbc = GraphDBClient(service('LATEST'))
-           >>> gdbc.connections(limit=10)
+            >>> from cimsparql.model import get_cim_model
+            >>> server_url = "127.0.0.1:7200"
+            >>> model = get_cim_model(server_url, "LATEST")
+            >>> model.connections(limit=10)
         """
         query = queries.connection_query(
             self.cim_version, rdf_types, region, sub_region, connectivity, nodes
         )
         if dry_run:
-            return self._query_with_header(query, limit)
-        return self._get_table_and_convert(query, limit, index="mrid")
+            return self.client.query_with_header(query, limit)
+        return self.get_table_and_convert(query, limit, index="mrid")
 
     def borders(
         self,
@@ -446,8 +387,8 @@ class CimModel(Model):
             market_optional,
         )
         if dry_run:
-            return self._query_with_header(query, limit)
-        return self._get_table_and_convert(query, limit, index="mrid")
+            return self.client.query_with_header(query, limit)
+        return self.get_table_and_convert(query, limit, index="mrid")
 
     def converters(
         self,
@@ -459,11 +400,15 @@ class CimModel(Model):
         dry_run: bool = False,
     ) -> Union[pd.DataFrame, str]:
         query = queries.converters(
-            region, sub_region, self.in_prefixes(converter_types), nodes, sequence_numbers
+            region,
+            sub_region,
+            self.client.prefixes.in_prefixes(converter_types),
+            nodes,
+            sequence_numbers,
         )
         if dry_run:
-            return self._query_with_header(query)
-        return self._get_table_and_convert(query, index="mrid")
+            return self.client.query_with_header(query)
+        return self.get_table_and_convert(query, index="mrid")
 
     def transformers_connected_to_converter(
         self,
@@ -482,11 +427,11 @@ class CimModel(Model):
 
         """
         query = queries.transformers_connected_to_converter(
-            region, sub_region, self.in_prefixes(converter_types)
+            region, sub_region, self.client.prefixes.in_prefixes(converter_types)
         )
         if dry_run:
-            return self._query_with_header(query)
-        return self._get_table_and_convert(query, index="converter_mrid")
+            return self.client.query_with_header(query)
+        return self.get_table_and_convert(query, index="converter_mrid")
 
     def ac_lines(
         self,
@@ -517,14 +462,14 @@ class CimModel(Model):
            dry_run: return string with sql query
 
         Example:
-           >>> from cimsparql.graphdb import GraphDBClient
-           >>> from cimsparql.url import service
-           >>> gdbc = GraphDBClient(service('LATEST'))
-           >>> gdbc.ac_lines(limit=10)
+            >>> from cimsparql.model import get_cim_model
+            >>> server_url = "127.0.0.1:7200"
+            >>> model = get_cim_model(server_url, "LATEST")
+            >>> model.ac_lines(limit=10)
         """
         query = line_queries.ac_line_query(
             self.cim_version,
-            self.ns["cim"],
+            self.client.prefixes.ns["cim"],
             region,
             sub_region,
             connectivity,
@@ -538,8 +483,8 @@ class CimModel(Model):
             length,
         )
         if dry_run:
-            return self._query_with_header(query, limit)
-        ac_lines = self._get_table_and_convert(query, limit=limit)
+            return self.client.query_with_header(query, limit)
+        ac_lines = self.get_table_and_convert(query, limit=limit)
         if temperatures is not None:
             for temperature in temperatures:
                 column = f"{sup.negpos(temperature)}_{abs(temperature)}_factor"
@@ -572,10 +517,10 @@ class CimModel(Model):
            dry_run: return string with sql query
 
         Example:
-           >>> from cimsparql.graphdb import GraphDBClient
-           >>> from cimsparql.url import service
-           >>> gdbc = GraphDBClient(service('LATEST'))
-           >>> gdbc.series_compensators(limit=10)
+            >>> from cimsparql.model import get_cim_model
+            >>> server_url = "127.0.0.1:7200"
+            >>> model = get_cim_model(server_url, "LATEST")
+            >>> model.series_compensators(limit=10)
         """
         query = line_queries.series_compensator_query(
             self.cim_version,
@@ -590,8 +535,8 @@ class CimModel(Model):
             impedance,
         )
         if dry_run:
-            return self._query_with_header(query, limit)
-        return self._get_table_and_convert(query, limit=limit)
+            return self.client.query_with_header(query, limit)
+        return self.get_table_and_convert(query, limit=limit)
 
     def transformers(
         self,
@@ -619,17 +564,17 @@ class CimModel(Model):
            dry_run: return string with sql query
 
         Example:
-           >>> from cimsparql.graphdb import GraphDBClient
-           >>> from cimsparql.url import service
-           >>> gdbc = GraphDBClient(service('LATEST'))
-           >>> gdbc.transformers(limit=10)
+            >>> from cimsparql.model import get_cim_model
+            >>> server_url = "127.0.0.1:7200"
+            >>> model = get_cim_model(server_url, "LATEST")
+            >>> model.transformers(limit=10)
         """
         query = queries.transformer_query(
             region, sub_region, connectivity, rates, network_analysis, with_market, impedance
         )
         if dry_run:
-            return self._query_with_header(query, limit)
-        return self._get_table_and_convert(query, limit=limit)
+            return self.client.query_with_header(query, limit)
+        return self.get_table_and_convert(query, limit=limit)
 
     def two_winding_transformers(
         self,
@@ -659,10 +604,10 @@ class CimModel(Model):
            dry_run: return string with sql query
 
         Example:
-           >>> from cimsparql.graphdb import GraphDBClient
-           >>> from cimsparql.url import service
-           >>> gdbc = GraphDBClient(service('LATEST'))
-           >>> gdbc.two_winding_transformers(limit=10)
+            >>> from cimsparql.model import get_cim_model
+            >>> server_url = "127.0.0.1:7200"
+            >>> model = get_cim_model(server_url, "LATEST")
+            >>> model.two_winding_transformers(limit=10)
         """
         query = queries.two_winding_transformer_query(
             region,
@@ -678,8 +623,8 @@ class CimModel(Model):
             self.cim_version,
         )
         if dry_run:
-            return self._query_with_header(query, limit)
-        return self._get_table_and_convert(query, limit=limit)
+            return self.client.query_with_header(query, limit)
+        return self.get_table_and_convert(query, limit=limit)
 
     def three_winding_transformers(
         self,
@@ -709,10 +654,10 @@ class CimModel(Model):
            dry_run: return string with sql query
 
         Example:
-           >>> from cimsparql.graphdb import GraphDBClient
-           >>> from cimsparql.url import service
-           >>> gdbc = GraphDBClient(service('LATEST'))
-           >>> gdbc.two_winding_transformers(limit=10)
+            >>> from cimsparql.model import get_cim_model
+            >>> server_url = "127.0.0.1:7200"
+            >>> model = get_cim_model(server_url, "LATEST")
+            >>> model.two_winding_transformers(limit=10)
         """
         query = queries.three_winding_transformer_query(
             region,
@@ -728,8 +673,8 @@ class CimModel(Model):
             self.cim_version,
         )
         if dry_run:
-            return self._query_with_header(query, limit)
-        return self._get_table_and_convert(query, limit=limit)
+            return self.client.query_with_header(query, limit)
+        return self.get_table_and_convert(query, limit=limit)
 
     def disconnected(
         self, index: Optional[str] = None, limit: Optional[int] = None, dry_run: bool = False
@@ -743,8 +688,8 @@ class CimModel(Model):
         """
         query = ssh_queries.disconnected(self.cim_version)
         if dry_run:
-            return self._query_with_header(query, limit)
-        return self.get_table(query, index=index, limit=limit)
+            return self.client.query_with_header(query, limit)
+        return self.get_table_and_convert(query, index=index, limit=limit)
 
     def ssh_synchronous_machines(
         self, limit: Optional[int] = None, dry_run: bool = False
@@ -757,8 +702,8 @@ class CimModel(Model):
         """
         query = ssh_queries.synchronous_machines()
         if dry_run:
-            return self._query_with_header(query, limit)
-        return self._get_table_and_convert(query, limit, index="mrid")
+            return self.client.query_with_header(query, limit)
+        return self.get_table_and_convert(query, limit, index="mrid")
 
     def ssh_load(
         self,
@@ -775,11 +720,11 @@ class CimModel(Model):
 
         """
         if rdf_types is None:
-            rdf_types = ["cim:ConformLoad", "cim:NonConformLoad"]
+            rdf_types = ["cim:ConformLoad", "cim:NonConformLoad", "cim:EnergyConsumer"]
         query = ssh_queries.load(rdf_types)
         if dry_run:
-            return self._query_with_header(query, limit)
-        return self._get_table_and_convert(query, limit, index="mrid")
+            return self.client.query_with_header(query, limit)
+        return self.get_table_and_convert(query, limit, index="mrid")
 
     def ssh_generating_unit(
         self,
@@ -797,11 +742,13 @@ class CimModel(Model):
 
         """
         if rdf_types is None:
-            rdf_types = [f"cim:{unit}GeneratingUnit" for unit in GeneratorTypes]
+            rdf_types = ["cim:GeneratingUnit"] + [
+                f"cim:{unit}GeneratingUnit" for unit in GeneratorTypes
+            ]
         query = ssh_queries.generating_unit(rdf_types)
         if dry_run:
-            return self._query_with_header(query, limit)
-        return self._get_table_and_convert(query, limit, index="mrid")
+            return self.client.query_with_header(query, limit)
+        return self.get_table_and_convert(query, limit, index="mrid")
 
     def terminal(
         self, limit: Optional[int] = None, dry_run: bool = False
@@ -814,8 +761,8 @@ class CimModel(Model):
         """
         query = tp_queries.terminal(self.cim_version)
         if dry_run:
-            return self._query_with_header(query, limit)
-        return self._get_table_and_convert(query, limit, index="mrid")
+            return self.client.query_with_header(query, limit)
+        return self.get_table_and_convert(query, limit, index="mrid")
 
     def topological_node(
         self, limit: Optional[int] = None, dry_run: bool = False
@@ -828,8 +775,8 @@ class CimModel(Model):
         """
         query = tp_queries.topological_node()
         if dry_run:
-            return self._query_with_header(query, limit)
-        return self._get_table_and_convert(query, limit, index="mrid")
+            return self.client.query_with_header(query, limit)
+        return self.get_table_and_convert(query, limit, index="mrid")
 
     def powerflow(
         self,
@@ -846,8 +793,8 @@ class CimModel(Model):
         """
         query = sv_queries.powerflow(power)
         if dry_run:
-            return self._query_with_header(query, limit)
-        return self._get_table_and_convert(query, limit, index="mrid")
+            return self.client.query_with_header(query, limit)
+        return self.get_table_and_convert(query, limit, index="mrid")
 
     def voltage(
         self,
@@ -864,8 +811,8 @@ class CimModel(Model):
         """
         query = sv_queries.voltage(voltage_vars)
         if dry_run:
-            return self._query_with_header(query, limit)
-        return self._get_table_and_convert(query, limit, index="mrid")
+            return self.client.query_with_header(query, limit)
+        return self.get_table_and_convert(query, limit, index="mrid")
 
     def tapstep(
         self, limit: Optional[int] = None, dry_run: bool = False
@@ -878,8 +825,8 @@ class CimModel(Model):
         """
         query = sv_queries.tapstep()
         if dry_run:
-            return self._query_with_header(query, limit)
-        return self._get_table_and_convert(query, limit, index="mrid")
+            return self.client.query_with_header(query, limit)
+        return self.get_table_and_convert(query, limit, index="mrid")
 
     @property
     def regions(self) -> pd.DataFrame:
@@ -889,10 +836,61 @@ class CimModel(Model):
            regions: List of regions in database
 
         Example:
-           >>> from cimsparql.graphdb import GraphDBClient
-           >>> from cimsparql.url import service
-           >>> gdbc = GraphDBClient(service('LATEST'))
-           >>> gdbc.regions
+            >>> from cimsparql.model import get_cim_model
+            >>> server_url = "127.0.0.1:7200"
+            >>> model = get_cim_model(server_url, "LATEST")
+            >>> model.regions
         """
-        query = queries.regions_query()
-        return self._get_table_and_convert(query, limit=None, index="mrid")
+        # TODO: Probably deprecate this property in the future. But keep for now.
+        # We need to find a solution for custom namespaces in queries
+        return self.get_regions()
+
+    def get_regions(self, with_sn_short_name: bool = True):
+        query = queries.regions_query(with_sn_short_name)
+        return self.get_table_and_convert(query, limit=None, index="mrid")
+
+    def add_mrid(
+        self, rdf_type: str, graph: Optional[str] = "?g", replace: Optional[Tuple[str, str]] = None
+    ):
+        """
+        Add cim:IdentifiedObject.mRID to all records. It is copied from rdf:about (or rdf:ID) if
+        replace is not specified
+
+        Args:
+            graph: Name of graph where mrids should be added. Note, mrid is added to all objects
+                in the graph.
+            rdf_type: RDF type where ID should be added
+            replace: Tuple with from/to replacements. Example: if the mrid is given by rdf:about
+                (or rdf:ID) where uuid is replace by an empty string pass ("uuid", "") as
+                replace argument
+        """
+
+        from_str, to_str = replace or ("", "")
+        query = (
+            f"INSERT {{GRAPH {graph} {{?s cim:IdentifiedObject.mRID ?mrid}}}} "
+            f"WHERE {{?s rdf:type {rdf_type}\n"
+            f'BIND(replace(str(?s), "{from_str}", "{to_str}") as ?mrid)}}'
+        )
+        query = self.client.query_with_header(query)
+        self.client.update_query(query)
+
+
+def get_cim_model(
+    server: str,
+    graphdb_repo: str,
+    graphdb_path: str = "services/pgm/equipment/",
+    protocol: str = "https",
+) -> CimModel:
+    """Get a CIM Model
+
+    Args:
+        server: graphdb server
+        graphdb_repo: query this repo
+        graphdb_path: Prepend with this path when graphdb_repo is LATEST
+        protocol: https or http
+    """
+    graphdb_path = graphdb_path if graphdb_repo == "LATEST" else ""
+    service_url = service(graphdb_repo, server, protocol, graphdb_path)
+    client = GraphDBClient(service_url)
+    mapper = TypeMapper(client)
+    return CimModel(mapper, client)
